@@ -27,12 +27,12 @@
 
 #define GIT_DIFF_STAGED(encoding_arg, context_arg, space_arg, old_name, new_name) \
 	"git", "diff-index", (encoding_arg), "--root", "--patch-with-stat", "-C", "-M", \
-		"--cached", "--diff-filter=ACDMRTXB", (context_arg), (space_arg), "HEAD", \
+		"--cached", "--diff-filter=ACDMRTXB", DIFF_ARGS, (context_arg), (space_arg), "HEAD", \
 		"--", (old_name), (new_name), NULL
 
 #define GIT_DIFF_UNSTAGED(encoding_arg, context_arg, space_arg, old_name, new_name) \
 	"git", "diff-files", (encoding_arg), "--root", "--patch-with-stat", "-C", "-M", \
-		(context_arg), (space_arg), "--", (old_name), (new_name), NULL
+		DIFF_ARGS, (context_arg), (space_arg), "--", (old_name), (new_name), NULL
 
 /* Don't show staged unmerged entries. */
 #define GIT_DIFF_STAGED_FILES(output_arg) \
