@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2014 Jonas Fonseca <jonas.fonseca@gmail.com>
+/* Copyright (c) 2006-2015 Jonas Fonseca <jonas.fonseca@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,18 +16,29 @@
 
 #include "tig/tig.h"
 
+typedef char repo_ref[SIZEOF_REF];
+typedef char repo_rev[SIZEOF_REV];
+typedef char repo_str[SIZEOF_STR];
+
+#define REPO_INFO(_) \
+	_(repo_ref, head) \
+	_(repo_rev, head_id) \
+	_(repo_ref, remote) \
+	_(repo_str, cdup) \
+	_(repo_str, prefix) \
+	_(repo_str, git_dir) \
+	_(bool,     is_inside_work_tree)
+
+#define REPO_INFO_FIELDS(type, name)	type name;
+
 struct repo_info {
-	char head[SIZEOF_REF];
-	char remote[SIZEOF_REF];
-	char cdup[SIZEOF_STR];
-	char prefix[SIZEOF_STR];
-	char git_dir[SIZEOF_STR];
-	bool is_inside_work_tree;
+	REPO_INFO(REPO_INFO_FIELDS)
 };
 
 extern struct repo_info repo;
 
-int load_repo_info(void);
+enum status_code load_repo_info(void);
+enum status_code load_repo_head(void);
 
 struct index_diff {
 	int staged;
