@@ -84,7 +84,8 @@ void string_copy_rev_from_commit_line(char *dst, const char *src);
 size_t string_expanded_length(const char *src, size_t srclen, size_t tabsize, size_t max_size);
 size_t string_expand(char *dst, size_t dstlen, const char *src, int srclen, int tabsize);
 
-char *chomp_string(char *name);
+char *string_trim_end(char *name);
+char *string_trim(char *name);
 
 bool PRINTF_LIKE(4, 5) string_nformat(char *buf, size_t bufsize, size_t *bufpos, const char *fmt, ...);
 
