@@ -50,6 +50,7 @@
 #include <sys/select.h>
 #include <unistd.h>
 #include <sys/time.h>
+#include <sys/file.h>
 #include <time.h>
 #include <fcntl.h>
 
@@ -59,6 +60,10 @@
 #include <langinfo.h>
 #include <iconv.h>
 
+#ifdef HAVE_EXECINFO_H
+#include <execinfo.h>
+#endif
+
 /* ncurses(3): Must be defined to have extended wide-character functions. */
 #define _XOPEN_SOURCE_EXTENDED 1
 
@@ -109,6 +114,7 @@
 #define SIZEOF_STR	1024	/* Default string size. */
 #define SIZEOF_REF	256	/* Size of symbolic or SHA1 ID. */
 #define SIZEOF_REV	41	/* Holds a SHA-1 and an ending NUL. */
+#define SIZEOF_CMD	8192	/* Default command string size. */
 
 /* This color name can be used to refer to the default term colors. */
 #define COLOR_DEFAULT	(-1)
