@@ -18,16 +18,15 @@
 #include "tig/prompt.h"
 
 static bool
-concat_argv(const char *argv[SIZEOF_ARG], char *buf, size_t buflen, const char *sep, bool quoted)
+concat_argv(const char *argv[], char *buf, size_t buflen, const char *sep, bool quoted)
 {
 	size_t bufpos, argc;
 
 	for (bufpos = 0, argc = 0; argv[argc]; argc++) {
 		const char *arg_sep = argc ? sep : "";
 		const char *arg = argv[argc];
-		int pos;
 
-		if (quoted && arg[(pos = strcspn(arg, " \t\""))]) {
+		if (quoted && arg[strcspn(arg, " \t\"")]) {
 			if (!string_nformat(buf, buflen, &bufpos, "%s\"", arg_sep))
 				return false;
 
@@ -37,7 +36,10 @@ concat_argv(const char *argv[SIZEOF_ARG], char *buf, size_t buflen, const char *
 
 				if (!string_nformat(buf, buflen, &bufpos, "%.*s%s", pos, arg, qesc))
 					return false;
-				arg += pos + 1;
+				if (!arg[pos])
+					break;
+				else
+					arg += pos + 1;
 			}
 
 			if (!string_nformat(buf, buflen, &bufpos, "\""))
@@ -53,6 +55,22 @@ concat_argv(const char *argv[SIZEOF_ARG], char *buf, size_t buflen, const char *
 	return true;
 }
 
+char *
+argv_to_string_alloc(const char *argv[], const char *sep)
+{
+	size_t i, size = 0;
+	char *buf;
+
+	for (i = 0; argv[i]; i++)
+		size += strlen(argv[i]);
+
+	buf = malloc(size + 1);
+	if (buf && argv_to_string(argv, buf, size + 1, sep))
+		return buf;
+	free(buf);
+	return NULL;
+}
+
 bool
 argv_to_string_quoted(const char *argv[SIZEOF_ARG], char *buf, size_t buflen, const char *sep)
 {
@@ -134,7 +152,7 @@ split_argv_string(const char *argv[SIZEOF_ARG], int *argc, char *cmd, bool remov
 		if (!arg)
 			break;
 		argv[(*argc)++] = arg;
-		cmd = chomp_string(cmd);
+		cmd = string_trim(cmd);
 	}
 
 	if (*argc < SIZEOF_ARG)
@@ -202,7 +220,7 @@ argv_contains(const char **argv, const char *arg)
 DEFINE_ALLOCATOR(argv_realloc, const char *, SIZEOF_ARG)
 
 bool
-argv_append(const char ***argv, const char *arg)
+argv_appendn(const char ***argv, const char *arg, size_t arglen)
 {
 	size_t argc = argv_size(*argv);
 	char *alloc;
@@ -213,7 +231,7 @@ argv_append(const char ***argv, const char *arg)
 	if (!argv_realloc(argv, argc, 2))
 		return false;
 
-	alloc = strdup(arg);
+	alloc = strndup(arg, arglen);
 
 	(*argv)[argc++] = alloc;
 	(*argv)[argc] = NULL;
@@ -221,6 +239,13 @@ argv_append(const char ***argv, const char *arg)
 	return alloc != NULL;
 }
 
+
+bool
+argv_append(const char ***argv, const char *arg)
+{
+	return argv_appendn(argv, arg, strlen(arg));
+}
+
 bool
 argv_append_array(const char ***dst_argv, const char *src_argv[])
 {
@@ -261,7 +286,7 @@ struct format_var {
 struct format_context {
 	struct format_var *vars;
 	size_t vars_size;
-	char buf[SIZEOF_STR];
+	char buf[SIZEOF_CMD];
 	size_t bufpos;
 	bool file_filter;
 };
@@ -427,7 +452,7 @@ argv_format(struct argv_env *argv_env, const char ***dst_argv, const char *src_a
 			if (file_filter && !argv_append_array(dst_argv, opt_file_args))
 				break;
 
-		} else if (!strcmp(arg, "%(diffargs)")) {
+		} else if (!strcmp(arg, DIFF_ARGS)) {
 			if (!format_append_argv(&format, dst_argv, opt_diff_options))
 				break;
 
@@ -510,6 +535,7 @@ argv_parse_rev_flag(const char *arg, struct rev_flags *rev_flags)
 		"--committer=",
 		"--date-order",
 		"--dense",
+		"--exclude=",
 		"--extended-regexp",
 		"--first-parent",
 		"--fixed-strings",
@@ -518,11 +544,11 @@ argv_parse_rev_flag(const char *arg, struct rev_flags *rev_flags)
 		"--glob=",
 		"--left-only",
 		"--max-parents=",
+		"--max-age=",
 		"--merge",
 		"--merges",
 		"--min-parents=",
 		"--no-max-parents",
-		"--no-merges",
 		"--no-min-parents",
 		"--no-walk",
 		"--perl-regexp",
@@ -549,6 +575,7 @@ argv_parse_rev_flag(const char *arg, struct rev_flags *rev_flags)
 		"-i",
 	};
 	static const char *no_graph[] = {
+		"--no-merges",
 		"--follow",
 	};
 	static const char *with_reflog[] = {
