@@ -21,38 +21,114 @@
 #include "tig/diff.h"
 #include "tig/draw.h"
 
-static bool
+static enum status_code
 diff_open(struct view *view, enum open_flags flags)
 {
 	const char *diff_argv[] = {
 		"git", "show", encoding_arg, "--pretty=fuller", "--root",
 			"--patch-with-stat", use_mailmap_arg(),
 			show_notes_arg(), diff_context_arg(), ignore_space_arg(),
-			"%(diffargs)", "%(cmdlineargs)", "--no-color", "%(commit)",
+			DIFF_ARGS, "%(cmdlineargs)", "--no-color", "%(commit)",
 			"--", "%(fileargs)", NULL
 	};
+	enum status_code code;
 
 	diff_save_line(view, view->private, flags);
 
-	return begin_update(view, NULL, diff_argv, flags);
+	code = begin_update(view, NULL, diff_argv, flags);
+	if (code != SUCCESS)
+		return code;
+
+	return diff_init_highlight(view, view->private);
+}
+
+enum status_code
+diff_init_highlight(struct view *view, struct diff_state *state)
+{
+	if (opt_diff_highlight) {
+		const char *argv[] = { opt_diff_highlight, NULL };
+		char * const env[] = { "GIT_CONFIG=/dev/null", NULL };
+		struct io io;
+
+		if (!io_exec(&io, IO_RP, view->dir, env, argv, view->io.pipe))
+			return error("Failed to run %s", opt_diff_highlight);
+
+		state->view_io = view->io;
+		view->io = io;
+		state->highlight = true;
+	}
+
+	return SUCCESS;
+}
+
+bool
+diff_done_highlight(struct diff_state *state)
+{
+	if (!state->highlight)
+		return true;
+	io_kill(&state->view_io);
+	return io_done(&state->view_io);
 }
 
 struct diff_stat_context {
 	const char *text;
 	enum line_type type;
+	bool skip;
 	size_t cells;
-	struct box_cell cell[10];
+	const char **cell_text;
+	struct box_cell cell[256];
 };
 
-static void
-diff_common_add_cell(struct diff_stat_context *context, size_t length)
+static bool
+diff_common_add_cell(struct diff_stat_context *context, size_t length, bool allow_empty)
 {
 	assert(ARRAY_SIZE(context->cell) > context->cells);
-	if (length == 0)
-		return;
+	if (!allow_empty && (length == 0))
+		return true;
+	if (context->skip && !argv_appendn(&context->cell_text, context->text, length))
+		return false;
 	context->cell[context->cells].length = length;
 	context->cell[context->cells].type = context->type;
 	context->cells++;
+	return true;
+}
+
+static struct line *
+diff_common_add_line(struct view *view, const char *text, enum line_type type, struct diff_stat_context *context)
+{
+	char *cell_text = context->cell_text ? argv_to_string_alloc(context->cell_text, "") : NULL;
+	const char *line_text = cell_text ? cell_text : text;
+	struct line *line = add_line_text_at(view, view->lines, line_text, type, context->cells);
+	struct box *box;
+
+	free(cell_text);
+	argv_free(context->cell_text);
+
+	if (!line)
+		return NULL;
+
+	box = line->data;
+	if (context->cells)
+		memcpy(box->cell, context->cell, sizeof(struct box_cell) * context->cells);
+	box->cells = context->cells;
+	return line;
+}
+
+static bool
+diff_common_add_cell_until(struct diff_stat_context *context, const char *s, enum line_type next_type)
+{
+	const char *sep = strstr(context->text, s);
+
+	if (sep == NULL)
+		return false;
+
+	if (!diff_common_add_cell(context, sep - context->text, false))
+		return false;
+
+	context->text = sep + (context->skip ? strlen(s) : 0);
+	context->type = next_type;
+
+	return true;
 }
 
 static bool
@@ -63,7 +139,7 @@ diff_common_read_diff_stat_part(struct diff_stat_context *context, char c, enum
 	if (sep == NULL)
 		return false;
 
-	diff_common_add_cell(context, sep - context->text);
+	diff_common_add_cell(context, sep - context->text, false);
 	context->text = sep;
 	context->type = next_type;
 
@@ -74,8 +150,6 @@ static struct line *
 diff_common_read_diff_stat(struct view *view, const char *text)
 {
 	struct diff_stat_context context = { text, LINE_DIFF_STAT };
-	struct line *line;
-	struct box *box;
 
 	diff_common_read_diff_stat_part(&context, '|', LINE_DEFAULT);
 	if (diff_common_read_diff_stat_part(&context, 'B', LINE_DEFAULT)) {
@@ -89,17 +163,9 @@ diff_common_read_diff_stat(struct view *view, const char *text)
 		diff_common_read_diff_stat_part(&context, '+', LINE_DIFF_ADD);
 		diff_common_read_diff_stat_part(&context, '-', LINE_DIFF_DEL);
 	}
-	diff_common_add_cell(&context, strlen(context.text));
+	diff_common_add_cell(&context, strlen(context.text), false);
 
-	line = add_line_text_at(view, view->lines, text, LINE_DIFF_STAT, context.cells);
-	if (!line)
-		return NULL;
-
-	box = line->data;
-	if (context.cells)
-		memcpy(box->cell, context.cell, sizeof(struct box_cell) * context.cells);
-	box->cells = context.cells;
-	return line;
+	return diff_common_add_line(view, text, LINE_DIFF_STAT, &context);
 }
 
 struct line *
@@ -112,7 +178,7 @@ diff_common_add_diff_stat(struct view *view, const char *text, size_t offset)
 
 	/* Ensure that '|' is present and the file name part contains
 	 * non-space characters. */
-	if (!pipe || pipe == data || strcspn(data, " ") == 0)
+	if (!pipe || pipe == data)
 		return NULL;
 
 	/* Detect remaining part of a diff stat line:
@@ -124,17 +190,33 @@ diff_common_add_diff_stat(struct view *view, const char *text, size_t offset)
 	 *	.../truncated file name  |   11 ++---
 	 *	binary add               |  Bin 0 -> 1234 bytes
 	 *	binary update            |  Bin 1234 -> 2345 bytes
+	 *	binary copy              |  Bin
 	 *	unmerged                 | Unmerged
 	 */
 	if ((data[len - 1] == '-' || data[len - 1] == '+') ||
-	    strstr(pipe, " 0") ||
-	    (strstr(pipe, "Bin") && strstr(pipe, "->")) ||
-	    strstr(pipe, "Unmerged") ||
+	    strstr(pipe, " 0") || strstr(pipe, "Bin") || strstr(pipe, "Unmerged") ||
 	    (data[len - 1] == '0' && (strstr(data, "=>") || !prefixcmp(data, "..."))))
 		return diff_common_read_diff_stat(view, text);
 	return NULL;
 }
 
+static bool
+diff_common_highlight(struct view *view, const char *text, enum line_type type)
+{
+	struct diff_stat_context context = { text, type, true };
+	enum line_type hi_type = type == LINE_DIFF_ADD
+				 ? LINE_DIFF_ADD_HIGHLIGHT : LINE_DIFF_DEL_HIGHLIGHT;
+	const char *codes[] = { "\x1b[7m", "\x1b[27m" };
+	const enum line_type types[] = { hi_type, type };
+	int i;
+
+	for (i = 0; diff_common_add_cell_until(&context, codes[i], types[i]); i = (i + 1) % 2)
+		;
+
+	diff_common_add_cell(&context, strlen(context.text), true);
+	return diff_common_add_line(view, text, type, &context);
+}
+
 bool
 diff_common_read(struct view *view, const char *data, struct diff_state *state)
 {
@@ -188,6 +270,9 @@ diff_common_read(struct view *view, const char *data, struct diff_state *state)
 		box->cell[box->cells++].type = LINE_DIFF_STAT;
 		return true;
 
+	} else if (state->highlight && strchr(data, 0x1b)) {
+		return diff_common_highlight(view, data, type);
+
 	} else if (type == LINE_PP_MERGE) {
 		state->combined_diff = true;
 	}
@@ -319,7 +404,7 @@ diff_read_describe(struct view *view, struct buffer *buffer, struct diff_state *
 	struct line *line = find_next_line_by_type(view, view->line, LINE_PP_REFS);
 
 	if (line && buffer) {
-		const char *ref = chomp_string(buffer->data);
+		const char *ref = string_trim(buffer->data);
 		const char *sep = !strcmp("Refs: ", box_text(line)) ? "" : ", ";
 
 		if (*ref && !append_line_format(view, line, "%s%s", sep, ref))
@@ -338,6 +423,11 @@ diff_read(struct view *view, struct buffer *buf, bool force_stop)
 		return diff_read_describe(view, buf, state);
 
 	if (!buf) {
+		if (!diff_done_highlight(state)) {
+			report("Failed run the diff-highlight program: %s", opt_diff_highlight);
+			return false;
+		}
+
 		/* Fall back to retry if no diff will be shown. */
 		if (view->lines == 0 && opt_file_args) {
 			int pos = argv_size(view->argv)
@@ -360,9 +450,10 @@ diff_read(struct view *view, struct buffer *buf, bool force_stop)
 
 		if (!state->adding_describe_ref && !ref_list_contains_tag(view->vid)) {
 			const char *describe_argv[] = { "git", "describe", view->vid, NULL };
+			enum status_code code = begin_update(view, NULL, describe_argv, OPEN_EXTRA);
 
-			if (!begin_update(view, NULL, describe_argv, OPEN_EXTRA)) {
-				report("Failed to load describe data");
+			if (code != SUCCESS) {
+				report("Failed to load describe data: %s", get_status_message(code));
 				return true;
 			}
 
@@ -529,6 +620,7 @@ diff_get_pathname(struct view *view, struct line *line)
 	struct line *header;
 	const char *dst;
 	const char *prefixes[] = { "diff --cc ", "diff --combined " };
+	const char *name;
 	int i;
 
 	header = find_prev_line_by_type(view, line, LINE_DIFF_HEADER);
@@ -545,11 +637,17 @@ diff_get_pathname(struct view *view, struct line *line)
 	if (!header)
 		return NULL;
 
+	name = box_text(header);
+	if (!prefixcmp(name, "+++ "))
+		name += STRING_SIZE("+++ ");
+
 	if (opt_diff_noprefix)
-		return box_text(header) + STRING_SIZE("+++ ");
+		return name;
 
 	/* Handle mnemonic prefixes, such as "b/" and "w/". */
-	return box_text(header) + STRING_SIZE("+++ b/");
+	if (!prefixcmp(name, "b/") || !prefixcmp(name, "w/"))
+		name += STRING_SIZE("b/");
+	return name;
 }
 
 enum request
