@@ -71,14 +71,13 @@ draw_chars(struct view *view, enum line_type type, const char *string, int lengt
 	}
 
 	set_view_attr(view, type);
-	if (len > 0) {
+	if (len > 0)
 		waddnstr(view->win, string, len);
 
-		if (trimmed && use_tilde) {
-			set_view_attr(view, LINE_DELIMITER);
-			waddch(view->win, '~');
-			col++;
-		}
+	if (trimmed && use_tilde) {
+		set_view_attr(view, LINE_DELIMITER);
+		waddstr(view->win, opt_truncation_delimiter ? opt_truncation_delimiter : "~");
+		col++;
 	}
 
 	view->col += col;
@@ -214,7 +213,7 @@ draw_field(struct view *view, enum line_type type, const char *text, int width,
 		int leftpad = max - textlen - 1;
 
 		if (leftpad > 0) {
-	    		if (draw_space(view, type, leftpad, leftpad))
+			if (draw_space(view, type, leftpad, leftpad))
 				return true;
 			max -= leftpad;
 			col += leftpad;;
@@ -308,7 +307,7 @@ static bool
 draw_lineno_custom(struct view *view, struct view_column *column, unsigned int lineno)
 {
 	char number[10];
-	unsigned long digits3 = column->width < 3 ? 3 : column->width;
+	unsigned long digits3 = MIN(9,MAX(3,column->width));
 	int max = MIN(VIEW_MAX_LEN(view), digits3);
 	char *text = NULL;
 	chtype separator = opt_line_graphics ? ACS_VLINE : '|';
@@ -321,7 +320,7 @@ draw_lineno_custom(struct view *view, struct view_column *column, unsigned int l
 	if (lineno == 1 || (lineno % interval) == 0) {
 		static char fmt[] = "%ld";
 
-		fmt[1] = '0' + (digits3 <= 9 ? digits3 : 1);
+		fmt[1] = '0' + digits3;
 		if (string_format(number, fmt, lineno))
 			text = number;
 	}
@@ -570,7 +569,7 @@ static void
 draw_view_line_search_result(struct view *view, unsigned int lineno)
 {
 	size_t bufsize = view->width * 4;
-	char *buf = malloc(bufsize);
+	char *buf = malloc(bufsize + 1);
 	regmatch_t pmatch[1];
 	regoff_t bufpos = 0;
 
