@@ -109,7 +109,7 @@ get_path_encoding(const char *path, struct encoding *default_encoding)
 
 	/* <path>: encoding: <encoding> */
 
-	if (!*path || !io_run_buf(check_attr_argv, buf, sizeof(buf), false)
+	if (!*path || !io_run_buf(check_attr_argv, buf, sizeof(buf), NULL, false)
 	    || !(encoding = strstr(buf, ENCODING_SEP)))
 		return default_encoding;
 
@@ -121,7 +121,7 @@ get_path_encoding(const char *path, struct encoding *default_encoding)
 			"file", "-I", "--", path, NULL
 		};
 
-		if (!*path || !io_run_buf(file_argv, buf, sizeof(buf), false)
+		if (!*path || !io_run_buf(file_argv, buf, sizeof(buf), NULL, false)
 		    || !(encoding = strstr(buf, CHARSET_SEP)))
 			return default_encoding;
 
@@ -131,6 +131,41 @@ get_path_encoding(const char *path, struct encoding *default_encoding)
 	return encoding_open(encoding);
 }
 
+/*
+ * Path manipulation.
+ */
+
+bool
+expand_path(char *dst, size_t dstlen, const char *src)
+{
+	if (!src)
+		return false;
+
+	if (src[0] == '~') {
+		/* constrain wordexp to tilde expansion only */
+		const char *ifs = getenv("IFS") ? getenv("IFS") : " \t\n";
+		wordexp_t we_result;
+		size_t metachar_pos;
+		char metachars[SIZEOF_STR];
+		char leading[SIZEOF_STR];
+
+		string_format(metachars, "%s%s", "/$|&;<>(){}`", ifs);
+		metachar_pos = strcspn(src, metachars);
+		if (src[metachar_pos] == '/' || src[metachar_pos] == 0) {
+			string_nformat(leading, metachar_pos + 1, NULL, "%s", src);
+			if (wordexp(leading, &we_result, WRDE_NOCMD))
+				return false;
+			string_nformat(dst, dstlen, NULL, "%s%s", we_result.we_wordv[0], src + metachar_pos);
+			wordfree(&we_result);
+			return true;
+		}
+	}
+
+	/* else */
+	string_ncopy_do(dst, dstlen, src, strlen(src));
+	return true;
+}
+
 /*
  * Executing external commands.
  */
@@ -179,7 +214,7 @@ io_done(struct io *io)
 	io_init(io);
 
 	while (pid > 0) {
-		int status;
+		int status = 0;
 		pid_t waiting = waitpid(pid, &status, 0);
 
 		if (waiting < 0) {
@@ -189,9 +224,7 @@ io_done(struct io *io)
 			return false;
 		}
 
-		if (WEXITSTATUS(status)) {
-			io->status = WEXITSTATUS(status);
-		}
+		io->status = WIFEXITED(status) ? WEXITSTATUS(status) : 0;
 
 		return waiting == pid &&
 		       !WIFSIGNALED(status) &&
@@ -216,16 +249,19 @@ open_trace(int devnull, const char *argv[])
 		int fd = open(trace_file, O_RDWR | O_CREAT | O_APPEND, 0666);
 		int i;
 
+		flock(fd, LOCK_EX);
 		for (i = 0; argv[i]; i++) {
 			if (write(fd, argv[i], strlen(argv[i])) == -1
 			    || write(fd, " ", 1) == -1)
 				break;
 		}
 		if (argv[i] || write(fd, "\n", 1) == -1) {
+			flock(fd, LOCK_UN);
 			close(fd);
 			return devnull;
 		}
 
+		flock(fd, LOCK_UN);
 		return fd;
 	}
 
@@ -268,7 +304,7 @@ io_exec(struct io *io, enum io_type type, const char *dir, char * const env[], c
 	if (dir && !strcmp(dir, argv[0]))
 		return io_open(io, "%s%s", dir, argv[1]);
 
-	if ((type == IO_RD || type == IO_WR) && pipe(pipefds) < 0) {
+	if ((type == IO_RD || type == IO_RP || type == IO_WR) && pipe(pipefds) < 0) {
 		io->error = errno;
 		return false;
 	} else if (type == IO_AP) {
@@ -288,9 +324,11 @@ io_exec(struct io *io, enum io_type type, const char *dir, char * const env[], c
 	} else {
 		if (type != IO_FG) {
 			int devnull = open("/dev/null", O_RDWR);
-			int readfd  = type == IO_WR ? pipefds[0] : devnull;
-			int writefd = (type == IO_RD || type == IO_AP)
-							? pipefds[1] : devnull;
+			int readfd  = type == IO_WR ? pipefds[0]
+				    : type == IO_RP ? custom
+				    : devnull;
+			int writefd = (type == IO_RD || type == IO_RP || type == IO_AP)
+				    ? pipefds[1] : devnull;
 			int errorfd = open_trace(devnull, argv);
 
 			/* Inject stdin given on the command line. */
@@ -314,7 +352,7 @@ io_exec(struct io *io, enum io_type type, const char *dir, char * const env[], c
 		}
 
 		if (dir && *dir && chdir(dir) == -1)
-			exit(errno);
+			_exit(errno);
 
 		if (env) {
 			int i;
@@ -325,7 +363,9 @@ io_exec(struct io *io, enum io_type type, const char *dir, char * const env[], c
 		}
 
 		execvp(argv[0], (char *const*) argv);
-		exit(errno);
+
+		close(STDOUT_FILENO);
+		_exit(errno);
 	}
 
 	if (pipefds[!!(type == IO_WR)] != -1)
@@ -426,7 +466,7 @@ io_memchr(struct buffer *buf, char *data, int c)
 DEFINE_ALLOCATOR(io_realloc_buf, char, BUFSIZ)
 
 static bool
-io_get_line(struct io *io, struct buffer *buf, int c, size_t *lineno, bool can_read)
+io_get_line(struct io *io, struct buffer *buf, int c, size_t *lineno, bool can_read, char eol_char)
 {
 	char *eol;
 	ssize_t readsize;
@@ -435,7 +475,7 @@ io_get_line(struct io *io, struct buffer *buf, int c, size_t *lineno, bool can_r
 		if (io->bufsize > 0) {
 			eol = memchr(io->bufpos, c, io->bufsize);
 
-			while (io->span && io->bufpos < eol && eol[-1] == '\\') {
+			while (eol_char && io->bufpos < eol && eol[-1] == eol_char) {
 				if (lineno)
 					(*lineno)++;
 				eol[-1] = eol[0] = ' ';
@@ -492,7 +532,7 @@ io_get_line(struct io *io, struct buffer *buf, int c, size_t *lineno, bool can_r
 bool
 io_get(struct io *io, struct buffer *buf, int c, bool can_read)
 {
-	return io_get_line(io, buf, c, NULL, can_read);
+	return io_get_line(io, buf, c, NULL, can_read, 0);
 }
 
 bool
@@ -537,7 +577,7 @@ io_read_buf(struct io *io, char buf[], size_t bufsize, bool allow_empty)
 	struct buffer result = {0};
 
 	if (io_get(io, &result, '\n', true)) {
-		result.data = chomp_string(result.data);
+		result.data = string_trim(result.data);
 		string_ncopy_do(buf, bufsize, result.data, strlen(result.data));
 	}
 
@@ -545,11 +585,11 @@ io_read_buf(struct io *io, char buf[], size_t bufsize, bool allow_empty)
 }
 
 bool
-io_run_buf(const char **argv, char buf[], size_t bufsize, bool allow_empty)
+io_run_buf(const char **argv, char buf[], size_t bufsize, const char *dir, bool allow_empty)
 {
 	struct io io;
 
-	return io_run(&io, IO_RD, NULL, NULL, argv) && io_read_buf(&io, buf, bufsize, allow_empty);
+	return io_run(&io, IO_RD, dir, NULL, argv) && io_read_buf(&io, buf, bufsize, allow_empty);
 }
 
 bool
@@ -577,18 +617,18 @@ io_load_file(struct io *io, const char *separators,
 	struct buffer buf;
 	enum status_code state = SUCCESS;
 
-	while (state == SUCCESS && io_get_line(io, &buf, '\n', lineno, true)) {
+	while (state == SUCCESS && io_get_line(io, &buf, '\n', lineno, true, '\\')) {
 		char *name;
 		char *value;
 		size_t namelen;
 		size_t valuelen;
 
-		name = chomp_string(buf.data);
+		name = string_trim(buf.data);
 		namelen = strcspn(name, separators);
 
 		if (name[namelen]) {
 			name[namelen] = 0;
-			value = chomp_string(name + namelen + 1);
+			value = string_trim(name + namelen + 1);
 			valuelen = strlen(value);
 
 		} else {
@@ -610,7 +650,6 @@ enum status_code
 io_load_span(struct io *io, const char *separators, size_t *lineno,
 	     io_read_fn read_property, void *data)
 {
-	io->span = true;
 	return io_load_file(io, separators, lineno, read_property, data);
 }
 
