@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2014 Jonas Fonseca <jonas.fonseca@gmail.com>
+/* Copyright (c) 2006-2015 Jonas Fonseca <jonas.fonseca@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,7 +16,6 @@
 #include "tig/argv.h"
 #include "tig/io.h"
 #include "tig/keys.h"
-#include "tig/util.h"
 
 struct keybinding {
 	enum request request;
@@ -26,6 +25,7 @@ struct keybinding {
 
 static struct keymap keymaps[] = {
 	{ "generic" },
+	{ "search" },
 #define VIEW_KEYMAP(id, name) { #name }
 	VIEW_INFO(VIEW_KEYMAP)
 };
@@ -33,11 +33,8 @@ static struct keymap keymaps[] = {
 static struct keymap *generic_keymap = keymaps;
 #define is_generic_keymap(keymap) ((keymap) == generic_keymap)
 
-struct keymap *
-get_keymap_by_index(int i)
-{
-	return 0 <= i && i < ARRAY_SIZE(keymaps) ? &keymaps[i] : NULL;
-}
+static struct keymap *search_keymap = keymaps + 1;
+#define is_search_keymap(keymap) ((keymap) == search_keymap)
 
 struct keymap *
 get_keymap(const char *name, size_t namelen)
@@ -52,18 +49,18 @@ get_keymap(const char *name, size_t namelen)
 }
 
 static bool
-keybinding_equals(struct keybinding *keybinding, struct key key[],
+keybinding_matches(const struct keybinding *keybinding, const struct key key[],
 		  size_t keys, bool *conflict_ptr)
 {
-	bool conflict = FALSE;
+	bool conflict = false;
 	int i;
 
-	if (keybinding->keys != keys)
-		return FALSE;
+	if (keybinding->keys < keys)
+		return false;
 
 	for (i = 0; i < keys; i++) {
-		struct key *key1 = &keybinding->key[i];
-		struct key *key2 = &key[i];
+		const struct key *key1 = &keybinding->key[i];
+		const struct key *key2 = &key[i];
 
 		if (key1->modifiers.control &&
 		    key1->modifiers.multibytes &&
@@ -74,27 +71,36 @@ keybinding_equals(struct keybinding *keybinding, struct key key[],
 			int c2 = key2->data.bytes[0];
 
 			if (ascii_toupper(c1) != ascii_toupper(c2))
-				return FALSE;
+				return false;
 			if (c1 != c2)
-				conflict = TRUE;
+				conflict = true;
 		} else {
 			if (memcmp(key1, key2, sizeof(*key1)))
-				return FALSE;
+				return false;
 		}
 	}
 
-	if (conflict_ptr)
+	if (conflict_ptr && keybinding->request != REQ_NONE)
 		*conflict_ptr = conflict;
-	return TRUE;
+	return true;
+}
+
+static bool
+keybinding_equals(const struct keybinding *keybinding, const struct key key[],
+		  size_t keys, bool *conflict_ptr)
+{
+	if (keybinding->keys != keys)
+		return false;
+	return keybinding_matches(keybinding, key, keys, conflict_ptr);
 }
 
 enum status_code
 add_keybinding(struct keymap *table, enum request request,
-	       struct key key[], size_t keys)
+	       const struct key key[], size_t keys)
 {
 	struct keybinding *keybinding;
 	char buf[SIZEOF_STR];
-	bool conflict = FALSE;
+	bool conflict = false;
 	size_t i;
 
 	for (i = 0; i < table->size; i++) {
@@ -126,22 +132,54 @@ add_keybinding(struct keymap *table, enum request request,
 	return SUCCESS;
 }
 
-/* Looks for a key binding first in the given map, then in the generic map, and
- * lastly in the default keybindings. */
-enum request
-get_keybinding(struct keymap *keymap, struct key key[], size_t keys)
+static enum request
+get_keybinding_in_keymap(const struct keymap *keymap, const struct key key[], size_t keys, int *matches)
 {
+	enum request request = REQ_UNKNOWN;
 	size_t i;
 
 	for (i = 0; i < keymap->size; i++)
-		if (keybinding_equals(keymap->data[i], key, keys, NULL))
-			return keymap->data[i]->request;
+		if (keybinding_matches(keymap->data[i], key, keys, NULL)) {
+			if (matches && keymap->data[i]->request != REQ_NONE)
+				(*matches)++;
+			/* Overriding keybindings, might have been added
+			 * at the end of the keymap so we need to
+			 * iterate all keybindings. */
+			if (keymap->data[i]->keys == keys)
+				request = keymap->data[i]->request;
+		}
 
-	for (i = 0; i < generic_keymap->size; i++)
-		if (keybinding_equals(generic_keymap->data[i], key, keys, NULL))
-			return generic_keymap->data[i]->request;
+	return request;
+}
 
-	return REQ_NONE;
+/* Looks for a key binding first in the given keymap, then in the generic keymap. */
+enum request
+get_keybinding(const struct keymap *keymap, const struct key key[], size_t keys, int *matches)
+{
+	enum request request = get_keybinding_in_keymap(keymap, key, keys, matches);
+
+	if (!is_search_keymap(keymap)) {
+		int generic_matches = 0;
+		enum request generic_request = get_keybinding_in_keymap(generic_keymap, key, keys, &generic_matches);
+
+		/* Include generic matches iff there are more than one
+		 * so unbound keys in the current keymap still override
+		 * generic keys while still ensuring that the key combo
+		 * handler continues to wait for more keys if there is
+		 * another possible match. E.g. while in `main` view:
+		 *
+		 *   bind generic q  quit  # 'q' will quit
+		 *   bind main    q  none  # 'q' will do nothing
+		 *   bind generic qa quit  # 'qa' will quit
+		 *   bind main    qn next  # 'qn' will move to next entry
+		 */
+		if (matches && (request == REQ_UNKNOWN || generic_matches > 1))
+			(*matches) += generic_matches;
+		if (request == REQ_UNKNOWN)
+			request = generic_request;
+	}
+
+	return request == REQ_NONE ? REQ_UNKNOWN : request;
 }
 
 
@@ -186,10 +224,25 @@ static const struct key_mapping key_mappings[] = {
 	{ "F10",	KEY_F(10) },
 	{ "F11",	KEY_F(11) },
 	{ "F12",	KEY_F(12) },
+	{ "F13",	KEY_F(13) },
+	{ "F14",	KEY_F(14) },
+	{ "F15",	KEY_F(15) },
+	{ "F16",	KEY_F(16) },
+	{ "F17",	KEY_F(17) },
+	{ "F18",	KEY_F(18) },
+	{ "F19",	KEY_F(19) },
 	{ "ScrollBack",	KEY_SR },
 	{ "SBack",	KEY_SR },
 	{ "ScrollFwd",	KEY_SF },
 	{ "SFwd",	KEY_SF },
+	{ "BackTab",	KEY_BTAB },
+	{ "ShiftTab",	KEY_BTAB },
+	{ "ShiftLeft",	KEY_SLEFT },
+	{ "ShiftRight",	KEY_SRIGHT },
+	{ "ShiftDelete",	KEY_SDC },
+	{ "ShiftDel",	KEY_SDC },
+	{ "ShiftHome",	KEY_SHOME },
+	{ "ShiftEnd",	KEY_SEND },
 };
 
 static const struct key_mapping *
@@ -270,13 +323,6 @@ get_key_value(const char **name_ptr, struct key *key)
 			if (mapping->value == '<')
 				return parse_key_value(key, name_ptr, 0, "<", end);
 
-			if (mapping->value == KEY_ESC) {
-				size_t offset = (end - name) + 1;
-
-				key->modifiers.escape = 1;
-				return parse_key_value(key, name_ptr, offset, NULL, NULL);
-			}
-
 			*name_ptr = end + 1;
 			key->data.value = mapping->value;
 			return SUCCESS;
@@ -295,7 +341,7 @@ get_key_value(const char **name_ptr, struct key *key)
 }
 
 const char *
-get_key_name(const struct key key[], size_t keys)
+get_key_name(const struct key key[], size_t keys, bool quote_comma)
 {
 	static char buf[SIZEOF_STR];
 	size_t pos = 0;
@@ -306,27 +352,31 @@ get_key_name(const struct key key[], size_t keys)
 		const char *name = multibytes ? key[i].data.bytes : "";
 		const char *start = "";
 		const char *end = "";
+		bool use_symbolic;
 
-		if (key[i].modifiers.escape) {
-			start = "<Esc>";
-		} else if (key[i].modifiers.control) {
+		if (key[i].modifiers.control) {
 			start = "<Ctrl-";
 			end = ">";
-		} else if (*name == ',') {
+		} else if (*name == ',' && quote_comma) {
 			/* Quote commas so they stand out in the help view. */
 			start = "'";
 			end = "'";
 		}
 
 		/* Use symbolic name for spaces so they are readable. */
-		if (!*name || *name == ' ') {
+		use_symbolic = !*name || *name == ' ';
+		/* When listing keys for :save-options quote illegal characters. */
+		if (!quote_comma && (*name == '<' || *name == '#'))
+			use_symbolic = true;
+
+		if (use_symbolic) {
 			int value = *name ? *name : key[i].data.value;
 			int j;
 
 			name = "<?>";
 			for (j = 0; j < ARRAY_SIZE(key_mappings); j++)
 				if (key_mappings[j].value == value) {
-					start = key[i].modifiers.escape ? "<Esc><" : "<";
+					start = "<";
 					end = ">";
 					name = key_mappings[j].name;
 					break;
@@ -341,34 +391,34 @@ get_key_name(const struct key key[], size_t keys)
 }
 
 static bool
-append_key(char *buf, size_t *pos, const struct keybinding *keybinding)
+append_key(char *buf, size_t *pos, const struct keybinding *keybinding, bool all)
 {
 	const char *sep = *pos > 0 ? ", " : "";
-	const char *keyname = get_key_name(keybinding->key, keybinding->keys);
+	const char *keyname = get_key_name(keybinding->key, keybinding->keys, all);
 
 	return string_nformat(buf, BUFSIZ, pos, "%s%s", sep, keyname);
 }
 
 static bool
 append_keymap_request_keys(char *buf, size_t *pos, enum request request,
-			   struct keymap *keymap, bool all)
+			   const struct keymap *keymap, bool all)
 {
 	int i;
 
 	for (i = 0; i < keymap->size; i++) {
 		if (keymap->data[i]->request == request) {
-			if (!append_key(buf, pos, keymap->data[i]))
-				return FALSE;
+			if (!append_key(buf, pos, keymap->data[i], all))
+				return false;
 			if (!all)
 				break;
 		}
 	}
 
-	return TRUE;
+	return true;
 }
 
 const char *
-get_keys(struct keymap *keymap, enum request request, bool all)
+get_keys(const struct keymap *keymap, enum request request, bool all)
 {
 	static char buf[BUFSIZ];
 	size_t pos = 0;
@@ -400,33 +450,50 @@ static size_t run_requests;
 
 DEFINE_ALLOCATOR(realloc_run_requests, struct run_request, 8)
 
-enum status_code
-add_run_request(struct keymap *keymap, struct key key[],
-		size_t keys, const char **argv)
-{
-	struct run_request *req;
-	struct run_request_flags flags = {};
+#define COMMAND_FLAGS ":!?@<+"
 
-	if (!strchr(":!?@<", *argv[0]))
-		return error("Unknown request name: %s", argv[0]);
+enum status_code
+parse_run_request_flags(struct run_request_flags *flags, const char **argv)
+{
+	if (!argv[0])
+		return error("No arguments");
+
+	if (!strchr(COMMAND_FLAGS, *argv[0]))
+		return error("Unknown command flag '%c'; expected one of %s", argv[0][0], COMMAND_FLAGS);
 
 	while (*argv[0]) {
 		if (*argv[0] == ':') {
-			flags.internal = 1;
+			flags->internal = 1;
 			argv[0]++;
 			break;
 		} else if (*argv[0] == '@') {
-			flags.silent = 1;
+			flags->silent = 1;
 		} else if (*argv[0] == '?') {
-			flags.confirm = 1;
+			flags->confirm = 1;
 		} else if (*argv[0] == '<') {
-			flags.exit = 1;
+			flags->exit = 1;
+		} else if (*argv[0] == '+') {
+			flags->echo = 1;
 		} else if (*argv[0] != '!') {
 			break;
 		}
 		argv[0]++;
 	}
 
+	return SUCCESS;
+}
+
+enum status_code
+add_run_request(struct keymap *keymap, const struct key key[],
+		size_t keys, const char **argv)
+{
+	struct run_request *req;
+	struct run_request_flags flags = {0};
+	enum status_code code = parse_run_request_flags(&flags, argv);
+
+	if (code != SUCCESS)
+		return code;
+
 	if (!realloc_run_requests(&run_request, run_requests, 1))
 		return ERROR_OUT_OF_MEMORY;
 
@@ -448,4 +515,143 @@ get_run_request(enum request request)
 	return &run_request[request - REQ_RUN_REQUESTS - 1];
 }
 
+const char *
+format_run_request_flags(const struct run_request *req)
+{
+	static char flags[8];
+	int flagspos = 0;
+
+	memset(flags, 0, sizeof(flags));
+
+	if (req->flags.internal)
+		flags[flagspos++] = ':';
+	else
+		flags[flagspos] = '!'; /* Optional, if other flags are defined */
+
+	if (req->flags.silent)
+	    flags[flagspos++] = '@';
+	if (req->flags.confirm)
+	    flags[flagspos++] = '?';
+	if (req->flags.exit)
+		flags[flagspos++] = '<';
+	if (req->flags.echo)
+		flags[flagspos++] = '+';
+	if (flagspos > 1)
+		flags[flagspos++] = 0;
+
+	return flags;
+}
+
+struct key_visitor_state {
+	key_visitor_fn visitor;
+	void *data;
+	struct keymap *keymap;
+	bool combine_keys;
+	const char *group;
+};
+
+static bool
+foreach_key_visit(struct key_visitor_state *state, const char *group,
+		  enum request request,
+		  const struct request_info *req_info, const struct run_request *run_req)
+{
+	struct keymap *keymap = state->keymap;
+	int i;
+
+	if (state->group == group)
+		group = NULL;
+
+	if (state->combine_keys) {
+		const char *key = get_keys(keymap, request, true);
+
+		if (!key || !*key)
+			return true;
+
+		if (group)
+			state->group = group;
+		return state->visitor(state->data, group, keymap, request,
+				      key, req_info, run_req);
+	}
+
+	for (i = 0; i < keymap->size; i++) {
+		if (keymap->data[i]->request == request) {
+			struct keybinding *keybinding = keymap->data[i];
+			const char *key = get_key_name(keybinding->key, keybinding->keys, false);
+
+			if (!key || !*key)
+				continue;
+
+			if (!state->visitor(state->data, group, keymap, request,
+					    key, req_info, run_req))
+				return false;
+
+			if (group)
+				state->group = group;
+			group = NULL;
+		}
+	}
+
+	return true;
+}
+
+static bool
+foreach_key_request(void *data, const struct request_info *req_info, const char *group)
+{
+	struct key_visitor_state *state = data;
+
+	if (req_info->request == REQ_NONE)
+		return true;
+
+	return foreach_key_visit(state, group, req_info->request, req_info, NULL);
+}
+
+static bool
+foreach_key_run_request(struct key_visitor_state *state, bool internal, bool toggles)
+{
+	struct keymap *keymap = state->keymap;
+	const char *group = !internal ?	"External commands:" :
+			    toggles ?	"Option toggling:" :
+					"Internal commands:";
+	enum request request = REQ_RUN_REQUESTS + 1;
+
+	for (; true; request++) {
+		struct run_request *req = get_run_request(request);
+		const char *key;
+
+		if (!req)
+			break;
+
+		if (req->flags.internal != !!internal ||
+		    req->keymap != keymap ||
+		    !*(key = get_keys(keymap, request, true)))
+			continue;
+
+		if (toggles != !strcmp(req->argv[0], "toggle"))
+			continue;
+
+		if (!foreach_key_visit(state, group, request, NULL, req))
+			return false;
+	}
+
+	return true;
+}
+
+bool
+foreach_key(key_visitor_fn visitor, void *data, bool combine_keys)
+{
+	int i;
+
+	for (i = 0; i < ARRAY_SIZE(keymaps); i++) {
+		struct key_visitor_state state = { visitor, data, &keymaps[i], combine_keys };
+
+		if (!foreach_request(foreach_key_request, &state)
+		    || !foreach_key_run_request(&state, true, true)
+		    || !foreach_key_run_request(&state, true, false)
+		    || !foreach_key_run_request(&state, false, false))
+			return false;
+	}
+
+	return true;
+}
+
 /* vim: set ts=8 sw=8 noexpandtab: */
