@@ -622,6 +622,18 @@ parse_option(struct option_info *option, const char *prefix, const char *arg)
 		const char *alloc = NULL;
 		const char **value = option->value;
 
+		if (option->value == &opt_diff_highlight) {
+			bool enabled = false;
+
+			if (parse_bool(&enabled, arg) == SUCCESS) {
+				if (!enabled) {
+					*value = NULL;
+					return SUCCESS;
+				}
+				arg = "diff-highlight";
+			}
+		}
+
 		if (strlen(arg)) {
 			if (arg[0] == '"' && arg[strlen(arg) - 1] == '"')
 				alloc = strndup(arg + 1, strlen(arg + 1) - 1);
@@ -631,6 +643,18 @@ parse_option(struct option_info *option, const char *prefix, const char *arg)
 				return ERROR_OUT_OF_MEMORY;
 		}
 
+		if (!strcmp(name, "truncation-delimiter")) {
+			if (!strcmp(alloc, "utf-8") || !strcmp(alloc, "utf8")) {
+				alloc = strdup("⋯");
+				if (!alloc)
+					return ERROR_OUT_OF_MEMORY;
+			} else if (utf8_width_of(alloc, -1, -1) != 1) {
+				alloc = strdup("~");
+				if (!alloc)
+					return ERROR_OUT_OF_MEMORY;
+			}
+		}
+
 		free((void *) *value);
 		*value = alloc;
 		return SUCCESS;
@@ -667,12 +691,38 @@ parse_view_settings(struct view_column **view_column, const char *name_, const c
 	return parse_view_config(view_column, name, argv);
 }
 
+static enum status_code
+option_update(struct option_info *option, int argc, const char *argv[])
+{
+	enum status_code code;
+
+	if (option->seen)
+		return SUCCESS;
+
+	if (!strcmp(option->type, "const char **"))
+		return parse_args(option->value, argv + 2);
+
+	if (argc < 3)
+		return error("Invalid set command: set option = value");
+
+	if (!strcmp(option->type, "view_settings"))
+		return parse_view_settings(option->value, argv[0], argv + 2);
+
+	if (!strcmp(option->type, "struct ref_format **"))
+		return parse_ref_formats(option->value, argv + 2);
+
+	code = parse_option(option, "", argv[2]);
+	if (code == SUCCESS && argc != 3)
+		return error("Option %s only takes one value", argv[0]);
+
+	return code;
+}
+
 /* Wants: name = value */
 static enum status_code
 option_set_command(int argc, const char *argv[])
 {
 	struct option_info *option;
-	enum status_code code;
 
 	if (argc < 2)
 		return error("Invalid set command: set option = value");
@@ -681,28 +731,26 @@ option_set_command(int argc, const char *argv[])
 		return error("No value assigned to %s", argv[0]);
 
 	option = find_option_info(option_info, ARRAY_SIZE(option_info), "", argv[0]);
-	if (option) {
-		if (option->seen)
-			return SUCCESS;
+	if (option)
+		return option_update(option, argc, argv);
 
-		if (!strcmp(option->type, "const char **"))
-			return parse_args(option->value, argv + 2);
+	{
+		const char *obsolete[][2] = {
+			{ "status-untracked-dirs", "status-show-untracked-dirs" },
+		};
+		int index = find_remapped(obsolete, ARRAY_SIZE(obsolete), argv[0]);
 
-		if (argc < 3)
-			return error("Invalid set command: set option = value");
-
-		if (!strcmp(option->type, "view_settings"))
-			return parse_view_settings(option->value, argv[0], argv + 2);
-
-		if (!strcmp(option->type, "struct ref_format **"))
-			return parse_ref_formats(option->value, argv + 2);
-
-		code = parse_option(option, "", argv[2]);
-		if (code == SUCCESS && argc != 3)
-			return error("Option %s only takes one value", argv[0]);
-
-		return code;
+		if (index != -1) {
+			option = find_option_info(option_info, ARRAY_SIZE(option_info), "", obsolete[index][1]);
+			if (option) {
+				enum status_code code = option_update(option, argc, argv);
 
+				if (code != SUCCESS)
+					return code;
+				return error("%s has been renamed to %s",
+					     obsolete[index][0], obsolete[index][1]);
+			}
+		}
 	}
 
 	{
@@ -776,6 +824,7 @@ option_bind_command(int argc, const char *argv[])
 			{ "diff-context-down",		"diff-context" },
 			{ "diff-context-up",		"diff-context" },
 			{ "stage-next",			":/^@@" },
+			{ "status-untracked-dirs",	"status-show-untracked-dirs" },
 			{ "toggle-author",		"author" },
 			{ "toggle-changes",		"show-changes" },
 			{ "toggle-commit-order",	"show-commit-order" },
@@ -794,7 +843,7 @@ option_bind_command(int argc, const char *argv[])
 			{ "toggle-sort-field",		"sort-field" },
 			{ "toggle-sort-order",		"sort-order" },
 			{ "toggle-title-overflow",	"commit-title-overflow" },
-			{ "toggle-untracked-dirs",	"status-untracked-dirs" },
+			{ "toggle-untracked-dirs",	"status-show-untracked-dirs" },
 			{ "toggle-vertical-split",	"show-vertical-split" },
 		};
 		int alias;
@@ -841,14 +890,25 @@ static enum status_code
 option_source_command(int argc, const char *argv[])
 {
 	enum status_code code;
+	bool quiet = false;
 
-	if (argc < 1)
-		return error("Invalid source command: source path");
+	if ((argc < 1) || (argc > 2))
+		return error("Invalid source command: source [-q] <path>");
 
-	code = load_option_file(argv[0]);
+	if (argc == 2) {
+		if (!strcmp(argv[0], "-q"))
+			quiet = true;
+		else
+			return error("Invalid source option: %s", argv[0]);
+	}
+
+	code = load_option_file(argv[argc - 1]);
+
+	if (quiet)
+		return code == ERROR_FILE_DOES_NOT_EXIST ? 0 : code;
 
 	return code == ERROR_FILE_DOES_NOT_EXIST
-		? error("File does not exist: %s", argv[0]) : code;
+		? error("File does not exist: %s", argv[argc - 1]) : code;
 }
 
 enum status_code
@@ -925,26 +985,21 @@ load_option_file(const char *path)
 	if (!path || !strlen(path))
 		return SUCCESS;
 
-	if (!prefixcmp(path, "~/")) {
-		const char *home = getenv("HOME");
-
-		if (!home || !string_format(buf, "%s/%s", home, path + 2))
-			return error("Failed to expand ~ to user home directory");
-		path = buf;
-	}
+	if (!expand_path(buf, sizeof(buf), path))
+		return error("Failed to expand path: %s", path);
 
 	/* It's OK that the file doesn't exist. */
-	if (!io_open(&io, "%s", path)) {
+	if (!io_open(&io, "%s", buf)) {
 		/* XXX: Must return ERROR_FILE_DOES_NOT_EXIST so missing
 		 * system tigrc is detected properly. */
 		if (io_error(&io) == ENOENT)
 			return ERROR_FILE_DOES_NOT_EXIST;
-		return error("Error loading file %s: %s", path, io_strerror(&io));
+		return error("Error loading file %s: %s", buf, io_strerror(&io));
 	}
 
 	if (io_load_span(&io, " \t", &config.lineno, read_option, &config) != SUCCESS ||
 	    config.errors == true)
-		warn("Errors while loading %s.", path);
+		warn("Errors while loading %s.", buf);
 	return SUCCESS;
 }
 
@@ -1401,6 +1456,9 @@ read_repo_config_option(char *name, size_t namelen, char *value, size_t valuelen
 	else if (!strcmp(name, "diff.noprefix"))
 		parse_bool(&opt_diff_noprefix, value);
 
+	else if (!strcmp(name, "status.showUntrackedFiles"))
+		parse_bool(&opt_status_show_untracked_files, value);
+
 	else if (!prefixcmp(name, "tig.color."))
 		set_repo_config_option(name + 10, value, option_color_command);
 
