@@ -66,7 +66,7 @@ view_request(struct view *view, enum request request)
 	    view_has_flags(view, VIEW_SEND_CHILD_ENTER)) {
 		struct view *child = display[1];
 
-	    	if (forward_request_to_child(child, request)) {
+		if (forward_request_to_child(child, request)) {
 			view_request(child, request);
 			return REQ_NONE;
 		}
@@ -99,7 +99,7 @@ view_request(struct view *view, enum request request)
 	_('X', "commit ID display",		"id"), \
 	_('%', "file filtering",		"file-filter"), \
 	_('$', "commit title overflow display",	"commit-title-overflow"), \
-	_('d', "untracked directory info",	"status-untracked-dirs"), \
+	_('d', "untracked directory info",	"status-show-untracked-dirs"), \
 	_('|', "view split",			"vertical-split"), \
 
 static void
@@ -171,6 +171,8 @@ view_driver(struct view *view, enum request request)
 	case REQ_MOVE_HALF_PAGE_DOWN:
 	case REQ_MOVE_FIRST_LINE:
 	case REQ_MOVE_LAST_LINE:
+	case REQ_MOVE_WHEEL_DOWN:
+	case REQ_MOVE_WHEEL_UP:
 		move_view(view, request);
 		break;
 
@@ -292,16 +294,23 @@ view_driver(struct view *view, enum request request)
 		find_next(view, request);
 		break;
 
+	case REQ_MOVE_NEXT_MERGE:
+	case REQ_MOVE_PREV_MERGE:
+		report("Moving between merge commits is not supported by the %s view", view->name);
+		break;
+
 	case REQ_STOP_LOADING:
 		foreach_view(view, i) {
 			if (view->pipe)
 				report("Stopped loading the %s view", view->name),
 			end_update(view, true);
+			if (view_is_displayed(view))
+				update_view_title(view);
 		}
 		break;
 
 	case REQ_SHOW_VERSION:
-		report("tig-%s (built %s)", TIG_VERSION, __DATE__);
+		report("tig-%s", TIG_VERSION);
 		return true;
 
 	case REQ_SCREEN_REDRAW:
@@ -316,6 +325,7 @@ view_driver(struct view *view, enum request request)
 		report("Nothing to enter");
 		break;
 
+	case REQ_VIEW_CLOSE_NO_QUIT:
 	case REQ_VIEW_CLOSE:
 		/* XXX: Mark closed views by letting view->prev point to the
 		 * view itself. Parents to closed view should never be
@@ -327,6 +337,10 @@ view_driver(struct view *view, enum request request)
 			view->parent = NULL;
 			break;
 		}
+		if (request == REQ_VIEW_CLOSE_NO_QUIT) {
+			report("Can't close last remaining view");
+			break;
+		}
 		/* Fall-through */
 	case REQ_QUIT:
 		return false;
@@ -345,7 +359,7 @@ view_driver(struct view *view, enum request request)
  */
 
 static const char usage_string[] =
-"tig " TIG_VERSION " (" __DATE__ ")\n"
+"tig " TIG_VERSION " \n"
 "\n"
 "Usage: tig        [options] [revs] [--] [paths]\n"
 "   or: tig log    [options] [revs] [--] [paths]\n"
@@ -403,6 +417,23 @@ filter_options(const char *argv[], bool rev_parse)
 		return;
 	}
 
+	/* Add known revision arguments in opt_rev_args and use
+	 * git-rev-parse to filter out the remaining options.
+	 */
+	for (next = flags_pos = 0; argv[next]; next++) {
+		const char *arg = argv[next];
+
+		if (!strcmp(arg, "--"))
+			while (argv[next])
+				argv[flags_pos++] = argv[next++];
+		else if (argv_parse_rev_flag(arg, NULL))
+			argv_append(&opt_rev_args, arg);
+		else
+			argv[flags_pos++] = arg;
+	}
+
+	argv[flags_pos] = NULL;
+
 	filter_rev_parse(&opt_file_args, "--no-revs", "--no-flags", argv);
 	filter_rev_parse(&flags, "--flags", "--no-revs", argv);
 
@@ -421,17 +452,6 @@ filter_options(const char *argv[], bool rev_parse)
 		opt_cmdline_args = flags;
 	}
 
-	for (next = flags_pos = 0; argv[next]; next++) {
-		const char *arg = argv[next];
-
-		if (!strcmp(arg, "--all"))
-			argv_append(&opt_rev_args, arg);
-		else
-			argv[flags_pos++] = arg;
-	}
-
-	argv[flags_pos] = NULL;
-
 	filter_rev_parse(&opt_rev_args, "--symbolic", "--revs-only", argv);
 }
 
@@ -529,7 +549,7 @@ open_pager_mode(enum request request)
 	} else if (request == REQ_VIEW_DIFF) {
 		if (argv_contains(opt_rev_args, "--stdin"))
 			open_diff_view(NULL, OPEN_FORWARD_STDIN);
-		else              
+		else
 			open_diff_view(NULL, OPEN_STDIN);
 
 	} else {
@@ -578,11 +598,15 @@ handle_mouse_event(void)
 	if (!view)
 		return REQ_NONE;
 
+#ifdef BUTTON5_PRESSED
+	if (event.bstate & (BUTTON2_PRESSED | BUTTON5_PRESSED))
+#else
 	if (event.bstate & BUTTON2_PRESSED)
-		return REQ_SCROLL_WHEEL_DOWN;
+#endif
+		return opt_mouse_wheel_cursor ? REQ_MOVE_WHEEL_DOWN : REQ_SCROLL_WHEEL_DOWN;
 
 	if (event.bstate & BUTTON4_PRESSED)
-		return REQ_SCROLL_WHEEL_UP;
+		return opt_mouse_wheel_cursor ? REQ_MOVE_WHEEL_UP : REQ_SCROLL_WHEEL_UP;
 
 	if (event.bstate & BUTTON1_PRESSED) {
 		if (event.y == view->pos.lineno - view->pos.offset) {
@@ -603,6 +627,43 @@ handle_mouse_event(void)
 }
 #endif
 
+/*
+ * Error handling.
+ *
+ * Inspired by code from src/util.c in ELinks
+ * (f86be659718c0cd0a67f88b42f07044c23d0d028).
+ */
+
+#ifdef DEBUG
+void
+sigsegv_handler(int sig)
+{
+	if (die_callback)
+		die_callback();
+
+	fputs("Tig crashed!\n\n"
+	      "Please report this issue along with all info printed below to\n\n"
+	      "  https://github.com/jonas/tig/issues/new\n\n", stderr);
+
+	fputs("Tig version: ", stderr);
+	fputs(TIG_VERSION, stderr);
+	fputs("\n\n", stderr);
+
+#ifdef HAVE_EXECINFO_H
+	{
+		/* glibc way of doing this */
+		void *stack[20];
+		size_t size = backtrace(stack, 20);
+
+		backtrace_symbols_fd(stack, size, STDERR_FILENO);
+	}
+#endif
+
+	/* The fastest way OUT! */
+	abort();
+}
+#endif
+
 struct key_combo {
 	enum request request;
 	struct keymap *keymap;
@@ -661,11 +722,14 @@ main(int argc, const char *argv[])
 	enum request request = parse_options(argc, argv, pager_mode);
 	struct view *view;
 
-	prompt_init();
-
 	if (signal(SIGPIPE, SIG_IGN) == SIG_ERR)
 		die("Failed to setup signal handler");
 
+#ifdef DEBUG
+	if (signal(SIGSEGV, sigsegv_handler) == SIG_ERR)
+		die("Failed to setup signal handler");
+#endif
+
 	if (setlocale(LC_ALL, "")) {
 		codeset = nl_langinfo(CODESET);
 	}
@@ -674,6 +738,8 @@ main(int argc, const char *argv[])
 	die_if_failed(load_options(), "Failed to load user config.");
 	die_if_failed(load_git_config(), "Failed to load repo config.");
 
+	prompt_init();
+
 	/* Require a git repository unless when running in pager mode. */
 	if (!repo.git_dir[0] && request != REQ_VIEW_PAGER)
 		die("Not a git repository");
