@@ -180,7 +180,7 @@ get_relative_date(const struct time *time, char *buf, size_t buflen, bool compac
 const char *
 mkdate(const struct time *time, enum date date, bool local, const char *custom_format)
 {
-	static char buf[STRING_SIZE("2006-04-29 14:21") + 1];
+	static char buf[SIZEOF_STR];
 	struct tm tm;
 	const char *format;
 
@@ -239,7 +239,11 @@ mkfilesize(unsigned long size, enum file_size format)
 	return string_format(buf, "%ld", size) ? buf : NULL;
 }
 
-const struct ident unknown_ident = { "Unknown", "unknown@localhost" };
+const struct ident unknown_ident = {
+	"unknown@localhostUnknown",	// key
+	"Unknown",			// name
+	"unknown@localhost"		// email
+};
 
 int
 ident_compare(const struct ident *i1, const struct ident *i2)
