@@ -201,6 +201,14 @@ move_view(struct view *view, enum request request)
 		      ? view->lines - view->pos.lineno - 1 : view->height / 2;
 		break;
 
+	case REQ_MOVE_WHEEL_DOWN:
+		steps = opt_mouse_scroll;
+		break;
+
+	case REQ_MOVE_WHEEL_UP:
+		steps = -opt_mouse_scroll;
+		break;
+
 	case REQ_MOVE_UP:
 	case REQ_PREVIOUS:
 		steps = -1;
@@ -304,7 +312,7 @@ goto_id(struct view *view, const char *expr, bool from_start, bool save_search)
 		const char *rev_parse_argv[] = {
 			"git", "rev-parse", "--revs-only", rev, NULL
 		};
-		bool ok = rev && io_run_buf(rev_parse_argv, id, sizeof(id), true);
+		bool ok = rev && io_run_buf(rev_parse_argv, id, sizeof(id), NULL, true);
 
 		free(rev);
 		if (!ok) {
@@ -340,7 +348,7 @@ goto_id(struct view *view, const char *expr, bool from_start, bool save_search)
 		}
 	}
 
-	report("Unable to find commit '%s'", view->env->search);
+	report("Unable to find commit '%s'", id);
 }
 
 /*
@@ -523,14 +531,14 @@ view_exec(struct view *view, enum open_flags flags)
 	return io_exec(&view->io, IO_RD, view->dir, opt_env, view->argv, io_flags);
 }
 
-bool
+enum status_code
 begin_update(struct view *view, const char *dir, const char **argv, enum open_flags flags)
 {
 	bool extra = !!(flags & (OPEN_EXTRA));
 	bool refresh = flags & (OPEN_REFRESH | OPEN_PREPARED | OPEN_STDIN);
 
 	if (view_no_refresh(view, flags))
-		return true;
+		return SUCCESS;
 
 	if (view->pipe) {
 		if (extra)
@@ -545,17 +553,13 @@ begin_update(struct view *view, const char *dir, const char **argv, enum open_fl
 		bool file_filter = !view_has_flags(view, VIEW_FILE_FILTER) || opt_file_filter;
 
 		view->dir = dir;
-		if (!argv_format(view->env, &view->argv, argv, !view->prev, file_filter)) {
-			report("Failed to format %s arguments", view->name);
-			return false;
-		}
+		if (!argv_format(view->env, &view->argv, argv, !view->prev, file_filter))
+			return error("Failed to format %s arguments", view->name);
 	}
 
 	if (view->argv && view->argv[0] &&
-	    !view_exec(view, flags)) {
-		report("Failed to open %s view", view->name);
-		return false;
-	}
+	    !view_exec(view, flags))
+		return error("Failed to open %s view", view->name);
 
 	if (open_from_stdin(flags)) {
 		if (!io_open(&view->io, "%s", ""))
@@ -565,7 +569,7 @@ begin_update(struct view *view, const char *dir, const char **argv, enum open_fl
 	if (!extra)
 		setup_update(view, view->ops->id);
 
-	return true;
+	return SUCCESS;
 }
 
 bool
@@ -641,6 +645,9 @@ update_view_title(struct view *view)
 	WINDOW *window = view->title;
 	struct line *line = &view->line[view->pos.lineno];
 	unsigned int view_lines, lines;
+	int update_increment = view_has_flags(view, VIEW_LOG_LIKE | VIEW_GREP_LIKE)
+			       ? 100
+			       : view_has_flags(view, VIEW_DIFF_LIKE) ? 10 : 1;
 
 	assert(view_is_displayed(view));
 
@@ -661,7 +668,11 @@ update_view_title(struct view *view)
 		wprintw(window, " - %s %d of %zd",
 					   view->ops->type,
 					   line->lineno,
-					   view->lines - view->custom_lines);
+					   MAX(line->lineno,
+					       view->pipe
+					       ? update_increment *
+						 (size_t) ((view->lines - view->custom_lines) / update_increment)
+					       : view->lines - view->custom_lines));
 	}
 
 	if (view->pipe) {
@@ -719,10 +730,10 @@ maximize_view(struct view *view, bool redraw)
 	if (redraw) {
 		redraw_display(false);
 		report_clear();
-	}
 
-	if (view_has_flags(view, VIEW_FLEX_WIDTH))
-		load_view(view, NULL, OPEN_RELOAD);
+		if (view_has_flags(view, VIEW_FLEX_WIDTH))
+			load_view(view, NULL, OPEN_RELOAD);
+	}
 }
 
 void
@@ -743,6 +754,8 @@ load_view(struct view *view, struct view *prev, enum open_flags flags)
 	}
 
 	if (refresh) {
+		enum status_code code;
+
 		if (view->pipe)
 			end_update(view, true);
 		if (view->ops->private_size) {
@@ -755,8 +768,11 @@ load_view(struct view *view, struct view *prev, enum open_flags flags)
 			}
 		}
 
-		if (!view->ops->open(view, flags))
+		code = view->ops->open(view, flags);
+		if (code != SUCCESS) {
+			report("%s", get_status_message(code));
 			return;
+		}
 	}
 
 	if (prev) {
@@ -1427,7 +1443,6 @@ view_column_info_update(struct view *view, struct line *line)
 			break;
 
 		case VIEW_COLUMN_COMMIT_TITLE:
-			width = column->opt.commit_title.width;
 			break;
 
 		case VIEW_COLUMN_DATE:
@@ -1451,10 +1466,13 @@ view_column_info_update(struct view *view, struct line *line)
 			break;
 
 		case VIEW_COLUMN_LINE_NUMBER:
-			if (column_data.line_number)
-				width = count_digits(*column_data.line_number);
-			else
-				width = count_digits(view->lines);
+			width = column->opt.line_number.width;
+			if (!width) {
+				if (column_data.line_number)
+					width = count_digits(*column_data.line_number);
+				else
+					width = count_digits(view->lines);
+			}
 			if (width < 3)
 				width = 3;
 			break;
@@ -1471,11 +1489,9 @@ view_column_info_update(struct view *view, struct line *line)
 			break;
 
 		case VIEW_COLUMN_STATUS:
-			width = column->opt.status.width;
 			break;
 
 		case VIEW_COLUMN_TEXT:
-			width = column->opt.text.width;
 			break;
 		}
 
